#!/usr/bin/python
"""
PyTime - Python Clock

Copyright (C) 2001 James ''Wez''Weatherall

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""

try: import pylau
except ImportError: pass

import gtk
import sys
import time

def gdk_intern_atom(name):
    import _gtk
    return _gtk.gdk_atom_intern(name, 1)

def set_gdk_win_prop(window, property, type,
                     format, mode, data):
    import _gtk, GDK
    atom = _gtk.gdk_atom_intern(property, 1)
    if not int(atom): return 0
    int_atom = _gtk.gdk_atom_intern(type, 1)
    if not int(int_atom): return 0
    if mode=="REPLACE":gdk_mode = GDK.PROP_MODE_REPLACE
    elif mode=="APPEND":gdk_mode = GDK.PROP_MODE_APPEND
    elif mode=="PREPEND":gdk_mode = GDK.PROP_MODE_PREPEND
    try:
        window.property_change(atom, int_atom, format, gdk_mode, data)
        return 1
    except: return 0

class Clock(gtk.GtkWindow):
    def __init__(self, format, dock):
        # - Main window
        gtk.GtkWindow.__init__(self)
        self.set_title("PyTime")
        # *** Hack to get size right..
        self.__label = gtk.GtkLabel("00:00")
        butt=gtk.GtkButton()
        butt.add(self.__label)
        butt.connect("clicked", self.OnClicked)
        self.__menu = None
        self.add(butt)
        self.set_default_size(32, 16)
        self.connect("destroy", self.OnDestroy)
        self.show_all()
        self.SetDocked(dock)
        # - PopupMenu
        menu=gtk.GtkMenu()
        item_quit=gtk.GtkMenuItem("Quit")
        item_quit.connect("activate", self.destroy)
        menu.add(item_quit)
        item_dock=gtk.GtkCheckMenuItem("Dock")
        item_dock.set_active(self.__dock)
        item_dock.connect("activate", self.OnDockToggle)
        menu.add(item_dock)
        menu.show_all()
        self.__menu = menu
        # - Time display handling
        self.__fmt = format
        timeout = 1000
        if "H" in format: timeout = 3600000
        if "M" in format: timeout = 60000
        if "S" in format: timeout = 1000
        self.OnTimeout()
        self.__timeout = gtk.timeout_add(timeout, self.OnTimeout)
    def SetDocked(self, dock):
        self.realize()
        self.hide()
        if dock: type = [gdk_intern_atom("_NET_WM_WINDOW_TYPE_DOCK")]
        else: type = []
        set_gdk_win_prop(self.get_window(), "_NET_WM_WINDOW_TYPE",
                         "ATOM", 32, "REPLACE", type)
        self.set_default_size(32, 16)
        self.show()
        self.__dock = dock
    def OnDestroy(self, widget):
        if self.__menu:self.__menu.destroy()
        gtk.timeout_remove(self.__timeout)
        gtk.mainquit()
    def OnTimeout(self):
        text = time.strftime(self.__fmt,
                             time.localtime(time.time())) 
        self.__label.set_text(text)
        return 1
    def OnClicked(self, widget):
        self.__menu.popup(None, None, None,
                          0, 0)
    def OnDockToggle(self, widget):
        self.SetDocked(not self.__dock)

def Main(argv):
    dock = 0
    if "--dock" in argv:
        dock=1
        argv.remove("--dock")
    if len(argv) >= 2:
        format = argv[1]
    else:
        format = "%H:%M"
    clock = Clock(format, dock)
    gtk.mainloop()

if __name__ == "__main__":
    Main(sys.argv)
